
#install.packages(c('DESeq2', 'survival','survminer', 'dplyr','glmnet', 'ggplot2','GGally', 'rms', 'survivalROC', 'plotROC'))
library('DESeq2')
library('survival')
library('survminer')
library('dplyr')
library('glmnet')
library('ggplot2')
library('GGally')
library('rms')
library('survivalROC')
library('plotROC')


getwd()
setwd("C:/Users/Jason/Desktop/")
dir()

  #load data
target_gene_list= read.csv("log2fc1_interaction 1076 genes.txt",header = F) 
colnames(target_gene_list)="Symbol"
survival_data=read.csv("GBMLGG_survival.txt",sep = "\t")
counts_data=read.csv("HiSeqV2",sep = "\t")
condition_data=read.csv("TCGA.GBMLGG.sampleMap_GBMLGG_clinicalMatrix",sep = "\t")

survival_data=survival_data[,1:4]
counts_data_reduced=merge(counts_data,target_gene_list,by.x="sample",by.y="Symbol")
row.names(counts_data_reduced)=counts_data_reduced$sample
counts_data_reduced_transvered=as.data.frame(t(counts_data_reduced))
#counts_data_reduced_transvered=counts_data_reduced_transvered[-1,]
counts_data_reduced_transvered$sample=row.names(counts_data_reduced_transvered)



counts_data_reduced_transvered$sample
survival_data$sample
counts_data_reduced_transvered$sample<- gsub(counts_data_reduced_transvered$sample, pattern = "[:.:]", replacement = '_')
rownames(counts_data_reduced_transvered)=counts_data_reduced_transvered$sample
survival_data$sample=gsub(survival_data$sample, pattern = '-', replacement = '_')

survival_cancer=merge(survival_data,counts_data_reduced_transvered,by="sample")
rownames(survival_cancer)=survival_cancer$sample
survival_cancer$censoring_status=survival_cancer$OS
survival_cancer$overall_survival=survival_cancer$OS.time

gene_list_to_analysis=counts_data_reduced$sample


survival_cancer_row_name=rownames(survival_cancer)
survival_cancer=survival_cancer[,-c(1:2)]
survival_cancer=as.data.frame(lapply(survival_cancer,as.numeric))
rownames(survival_cancer)=survival_cancer_row_name
class(survival_cancer[,counts_data_reduced$sample][6,6])



#build counts matrix,   not needed here
res_DE <- results(dds_DE, alpha=0.05,contrast=c("sample_type","cancer","normal"))
resOrdered <- res_DE[order(res_DE$padj),]
resSig <- subset(resOrdered,padj < 0.01)
resSigAll <- subset(resSig, (log2FoldChange < (-2)|log2FoldChange > 2))
rld <- vst(dds_DE, blind = T)
expr_norm <- assay(rld)
DESeq_norm_vst_for_survival <- expr_norm[resSigAll@rownames,]

#build regression analysis related data,   not needed here
library('dplyr')
survival_cancer <- condition_table_cancer
#Alive=0, Dead=1
survival_cancer$censoring_status <- gsub(survival_cancer$censoring_status, pattern = 'Alive', replacement = '0') %>%
  gsub(pattern = 'Dead', replacement = '1') %>% as.numeric()
rownames(survival_cancer) <- survival_cancer$TCGA_IDs
survival_cancer <- cbind(survival_cancer, t(DESeq_norm_vst_for_survival)[survival_cancer$TCGA_IDs,])
#formula function in the next step will not recognize gene names like 'NKX1-2'. We have to change '-' to '_'.
colnames(survival_cancer) <- gsub(colnames(survival_cancer), pattern = '-', replacement = '_')



# gene list univariate analysis
#filter potential useful sig genes using univariate cox regression.
gene_list_to_analysis=as.list(gene_list_to_analysis)

uni_cox_in_bulk <- function(gene_list, survival_info_df){
  library('survival')
  #gene_list <- gsub(gene_list, pattern = '-', replacement = '_')
  uni_cox <- function(single_gene){
    formula <- as.formula(paste0('Surv(overall_survival, censoring_status)~', single_gene))
    surv_uni_cox <- summary(coxph(formula, data = survival_cancer))
    ph_hypothesis_p <- cox.zph(coxph(formula, data = survival_cancer))$table[1,3]
    if (surv_uni_cox$coefficients[,5]<0.05 & ph_hypothesis_p>0.05){  #get the pvalue
      single_cox_report <- data.frame('uni_cox_sig_genes'=single_gene,
                                      'beta'=surv_uni_cox$coefficients[,1],
                                      'Hazard_Ratio'=exp(surv_uni_cox$coefficients[,1]),
                                      'z_pvalue'=surv_uni_cox$coefficients[,5],
                                      'Wald_pvalue'=as.numeric(surv_uni_cox$waldtest[3]),
                                      'Likelihood_pvalue'=as.numeric(surv_uni_cox$logtest[3]))
      single_cox_report
    }
  }
  uni_cox_list <- lapply(gene_list, uni_cox)
  do.call(rbind, uni_cox_list)
}
uni_cox_df <- uni_cox_in_bulk(gene_list =gene_list_to_analysis, survival_info_df = survival_cancer)




#use LASSO regression analysis if univariate analysis gives too many genes
#about glmnet: x should be in format of matrix, and time&status in y should be in double format.
#survival_cancer=survival_cancer[-which(rowSums(is.na(survival_cancer)) > 0),]
#survival_cancer=survival_cancer[-which(survival_cancer$overall_survival== 0),]




x <- as.matrix(survival_cancer[,counts_data_reduced$sample])
y <- survival_cancer[,c('overall_survival', 'censoring_status')]
names(y) <- c('time', 'status')
y$time <- as.double(y$time)
y$status <- as.double(y$status)
y <- as.matrix(survival::Surv(y$time, y$status))
lasso_fit <- cv.glmnet(x, y, family='cox', type.measure = 'deviance')
coefficient <- coef(lasso_fit, s=lasso_fit$lambda.min)
Active.Index <- which(as.numeric(coefficient) != 0)
active.coefficients <- as.numeric(coefficient)[Active.Index]
sig_gene_multi_cox <- rownames(coefficient)[Active.Index]
length(sig_gene_multi_cox)
LASSO_sig_df=as.data.frame(cbind(sig_gene_multi_cox,active.coefficients))

length(Active.Index)

write.table(LASSO_sig_df,"LASSO_sig_df.txt",row.names = F,quote = F,sep = "\t")



#perform the multi-variates cox regression using qualified genes.

formula_for_multivariate <- as.formula(paste0('Surv(overall_survival, censoring_status)~', paste(sig_gene_multi_cox, sep = '', collapse = '+')))
multi_variate_cox <- coxph(formula_for_multivariate, data = survival_cancer)
#check if variances are supported by PH hypothesis.
ph_hypo_multi <- cox.zph(multi_variate_cox)
#The last row of the table records the test results on the GLOBAL model. Delete it.
ph_hypo_table <- ph_hypo_multi$table[-nrow(ph_hypo_multi$table),]
#Remove variances not supported by ph hypothesis and perform the 2nd regression.
formula_for_multivariate <- as.formula(paste0('Surv(overall_survival, censoring_status)~', paste(rownames(ph_hypo_table)[ph_hypo_table[,3]>0.05], sep = '', collapse = '+')))
multi_variate_cox_2 <- coxph(formula_for_multivariate, data = survival_cancer)

#check the co-linearity between samples.
correlation <- cor(survival_cancer[,rownames(ph_hypo_table)[ph_hypo_table[,3]>0.05]], method = 'pearson')
library('GGally')
ggpairs(survival_cancer[,rownames(ph_hypo_table)[ph_hypo_table[,3]>0.05]], 
        axisLabels = 'show')+
  theme_bw()+
  theme(panel.background = element_rect(colour = 'black', size=1, fill = 'white'), 
        panel.grid = element_blank())
library('rms')
vif <- rms::vif(multi_variate_cox_2)
#Some people said if the square root of VIF >2, they might be co-linear.
sqrt(vif) < 2

length(sqrt(vif))

# multi-cox forest plot
ggforest(model = multi_variate_cox_2, data = survival_cancer, 
          main = 'Hazard ratios of candidate genes', 
          cpositions = c(0.02, 0.2, 0.4),  #???????????????
          fontsize = 0.6)



#calculate the risk score of each sample.
survival_cancer$TCGA_IDs=ROWNAMES(survival_cancer)

riskscore <- function(survival_cancer_df, candidate_genes_for_cox, cox_report) {
  library('dplyr')
  risk_score_table <- survival_cancer_df[,candidate_genes_for_cox]
  for(each_sig_gene in colnames(risk_score_table)){
    risk_score_table$each_sig_gene <- risk_score_table[,each_sig_gene]*(summary(cox_report)$coefficients[each_sig_gene,1])
  }
  risk_score_table <- cbind(risk_score_table, 'total_risk_score'=exp(rowSums(risk_score_table))) %>%
    cbind(survival_cancer_df[,c('TCGA_IDs','overall_survival','censoring_status')])
  risk_score_table <- risk_score_table[,c('TCGA_IDs','overall_survival','censoring_status', candidate_genes_for_cox, 'total_risk_score')]
  risk_score_table
}

candidate_genes_for_cox2 <- c(rownames(ph_hypo_table)[ph_hypo_table[,3]>0.05])
risk_score_table_multi_cox2 <- riskscore(survival_cancer, final_gene_list, multi_variate_cox_2)

final_gene_list=c("ARHGAP11A","DRP2","HNRNPA3","IGFBP5","IP6K2","KLF10","KPNA4","NRP2","PAIP1","RCN1","SEMA5A")


#draw ROC and calculate AUC
multi_ROC <- function(time_vector, risk_score_table){
  library('survivalROC')
  single_ROC <- function(single_time){
    for_ROC <- survivalROC(Stime = risk_score_table$overall_survival,
                           status = risk_score_table$censoring_status,
                           marker = risk_score_table$total_risk_score,
                           predict.time = single_time, method = 'KM')
    data.frame('True_positive'=for_ROC$TP, 'False_positive'=for_ROC$FP, 
               'Cut_values'=for_ROC$cut.values, 'Time_point'=rep(single_time, length(for_ROC$TP)),
               'AUC'=rep(for_ROC$AUC, length(for_ROC$TP)))
  }
  multi_ROC_list <- lapply(time_vector, single_ROC)
  do.call(rbind, multi_ROC_list)
}
#We evaluate 11 AUCs between 3-5 years.
for_multi_ROC <- multi_ROC(time_vector = c(365*seq(3,5,0.2)), risk_score_table = risk_score_table_multi_cox2)


#visualization of the ROC curves of multiple time points.
AUC_max=max(for_multi_ROC$AUC)
for_multi_ROC[for_multi_ROC$AUC==AUC_max,]
AUC_max_time <- for_multi_ROC$Time_point[which(for_multi_ROC$AUC == AUC_max)]
AUC_max_time <- AUC_max_time[!duplicated(AUC_max_time)]
AUC_max_time <- AUC_max_time[length(AUC_max_time)]

pROC<-ggplot(for_multi_ROC, aes(x = False_positive, y = True_positive, label = Cut_values, color = Time_point)) + 
  geom_roc(labels = F, stat = 'identity', n.cuts = 0) + 
  geom_abline(slope = 1, intercept = 0, color = 'red', linetype = 2)+
  theme_bw()+
  theme(panel.background = element_rect(colour = 'black', size=1, fill = 'white'), 
        panel.grid = element_blank())+
  annotate("text",x = 0.75, y = 0.15,
           label = paste("AUC max = ", round(AUC_max, 2), '\n', 'AUC max time = ', AUC_max_time, ' days', sep = ''))
pROC

ggsave("filename.tiff", dpi=600)        
tiff("filename.tiff", width=, height=)
dev.off()




#K-M curve analysis
AUC_max <- max(for_multi_ROC$AUC)
#maybe AUCs are identical in different time points. So select the last time point indicating longer survival.
AUC_max_time <- for_multi_ROC$Time_point[which(for_multi_ROC$AUC == AUC_max)]
AUC_max_time <- AUC_max_time[!duplicated(AUC_max_time)]
AUC_max_time <- AUC_max_time[length(AUC_max_time)]
for_multi_ROC$Time_point <- as.factor(for_multi_ROC$Time_point)
#find the optimal cutoff value within the ROC curve of the optimal time point.
optimal_time_ROC_df <- for_multi_ROC[which(for_multi_ROC$Time_point == AUC_max_time),]
cut.off <- optimal_time_ROC_df$Cut_values[which.max(optimal_time_ROC_df$True_positive-optimal_time_ROC_df$False_positive)]
high_low <- (risk_score_table_multi_cox2$total_risk_score > cut.off)
high_low[high_low == TRUE] <- 'high'
high_low[high_low == FALSE] <- 'low'
risk_score_table_multi_cox2 <- cbind(risk_score_table_multi_cox2, high_low)

length(risk_score_table_multi_cox2[risk_score_table_multi_cox2$high_low=="low",]$TCGA_IDs)


#KM_plot generation.
library('survminer')
#first edit the status of patients with OS > AUC max time. (censoring status=0 (Alive), OS=365*5 days)
risk_score_table_multi_cox2$censoring_status[which(risk_score_table_multi_cox2$overall_survival > AUC_max_time)] <- 0
risk_score_table_multi_cox2$overall_survival[which(risk_score_table_multi_cox2$overall_survival > AUC_max_time)] <- AUC_max_time
fit_km <- survfit(Surv(overall_survival, censoring_status) ~high_low, data = risk_score_table_multi_cox2)     
ggsurvplot(fit_km, conf.int = F,pval = T,legend.title="risk score group",
           legend.labs=c(paste0("high risk"), paste0("low risk")), 
           risk.table = T, 
           palette = c('red','blue'), surv.median.line = 'hv')

























### validation in CGGA data

#load CGGA data
target_gene_list_2= as.data.frame(candidate_genes_for_cox2)

colnames(target_gene_list_2)="Symbol"
survival_data_2=read.csv("CGGA.mRNA_array_301_clinical.20200506.txt",sep = "\t")
colnames(survival_data_2)
survival_data_2=survival_data_2[,c("CGGA_ID","OS","Censor..alive.0..dead.1.")]


counts_data_2=read.csv("CGGA.mRNA_array_301_gene_level.20200506.txt",sep = "\t")
rownames(counts_data_2)=counts_data_2$Gene_Name
counts_data_2=counts_data_2[,-1]
counts_data_2=as.data.frame(t(counts_data_2))
counts_data_2=counts_data_2[,candidate_genes_for_cox2]
#counts_data_2=counts_data_2[,intersect(colnames(counts_data_2),candidate_genes_for_cox2)]


survival_cancer_cgga=cbind(survival_data_2,counts_data_2)

survival_cancer_cgga=survival_cancer_cgga[-which(rowSums(is.na(survival_cancer_cgga)) > 0),]
#survival_cancer_cgga=survival_cancer_cgga[-which(survival_cancer_cgga$OS== 0),]
survival_cancer_cgga$TCGA_IDs=survival_cancer_cgga$CGGA_ID
survival_cancer_cgga$overall_survival=survival_cancer_cgga$OS
survival_cancer_cgga$censoring_status=survival_cancer_cgga$Censor..alive.0..dead.1.



#calculate the risk score of each sample.
riskscore <- function(survival_cancer_df, candidate_genes_for_cox, cox_report) {
  library('dplyr')
  risk_score_table <- survival_cancer_df[,candidate_genes_for_cox]
  for(each_sig_gene in colnames(risk_score_table)){
    risk_score_table$each_sig_gene <- risk_score_table[,each_sig_gene]*(summary(cox_report)$coefficients[each_sig_gene,1])
  }
  risk_score_table <- cbind(risk_score_table, 'total_risk_score'=exp(rowSums(risk_score_table))) %>%
    cbind(survival_cancer_df[,c('TCGA_IDs','overall_survival','censoring_status')])
  risk_score_table <- risk_score_table[,c('TCGA_IDs','overall_survival','censoring_status', candidate_genes_for_cox, 'total_risk_score')]
  risk_score_table
}
risk_score_table_multi_cox_cgga <- riskscore(survival_cancer_cgga, final_gene_list, multi_variate_cox_2)




#draw ROC and calculate AUC
multi_ROC <- function(time_vector, risk_score_table){
  library('survivalROC')
  single_ROC <- function(single_time){
    for_ROC <- survivalROC(Stime = risk_score_table$overall_survival,
                           status = risk_score_table$censoring_status,
                           marker = risk_score_table$total_risk_score,
                           predict.time = single_time, method = 'KM')
    data.frame('True_positive'=for_ROC$TP, 'False_positive'=for_ROC$FP, 
               'Cut_values'=for_ROC$cut.values, 'Time_point'=rep(single_time, length(for_ROC$TP)),
               'AUC'=rep(for_ROC$AUC, length(for_ROC$TP)))
  }
  multi_ROC_list <- lapply(time_vector, single_ROC)
  do.call(rbind, multi_ROC_list)
}
#We evaluate 11 AUCs between 3-5 years.
for_multi_ROC_cgga <- multi_ROC(time_vector = c(365*seq(3,5,0.2)), risk_score_table = risk_score_table_multi_cox_cgga)


#visualization of the ROC curves of multiple time points.
AUC_max=max(for_multi_ROC_cgga$AUC)
AUC_max_time <- for_multi_ROC_cgga$Time_point[which(for_multi_ROC_cgga$AUC == AUC_max)]
AUC_max_time <- AUC_max_time[!duplicated(AUC_max_time)]
AUC_max_time <- AUC_max_time[length(AUC_max_time)]

pROC<-ggplot(for_multi_ROC_cgga, aes(x = False_positive, y = True_positive, label = Cut_values, color = Time_point)) + 
  geom_roc(labels = F, stat = 'identity', n.cuts = 0) + 
  geom_abline(slope = 1, intercept = 0, color = 'red', linetype = 2)+
  theme_bw()+
  theme(panel.background = element_rect(colour = 'black', size=1, fill = 'white'), 
        panel.grid = element_blank())+
  annotate("text",x = 0.75, y = 0.15,
           label = paste("AUC max = ", round(AUC_max, 2), '\n', 'AUC max time = ', AUC_max_time, ' days', sep = ''))
pROC

ggsave("filename.tiff", dpi=600)        
tiff("filename.tiff", width=, height=)
dev.off()




#K-M curve analysis
AUC_max <- max(for_multi_ROC$AUC)
#maybe AUCs are identical in different time points. So select the last time point indicating longer survival.
AUC_max_time <- for_multi_ROC$Time_point[which(for_multi_ROC$AUC == AUC_max)]
AUC_max_time <- AUC_max_time[!duplicated(AUC_max_time)]
AUC_max_time <- AUC_max_time[length(AUC_max_time)]
for_multi_ROC$Time_point <- as.factor(for_multi_ROC$Time_point)

#find the optimal cutoff value within the ROC curve of the optimal time point.
optimal_time_ROC_df <- for_multi_ROC_cgga[which(for_multi_ROC_cgga$Time_point == AUC_max_time),]
cut.off <- optimal_time_ROC_df$Cut_values[which.max(optimal_time_ROC_df$True_positive-optimal_time_ROC_df$False_positive)]
high_low <- (risk_score_table_multi_cox_cgga$total_risk_score > cut.off)
high_low[high_low == TRUE] <- 'high'
high_low[high_low == FALSE] <- 'low'
risk_score_table_multi_cox_cgga <- cbind(risk_score_table_multi_cox_cgga, high_low)

length(risk_score_table_multi_cox_cgga[risk_score_table_multi_cox_cgga$high_low=="low",]$TCGA_IDs)


#KM_plot generation.
library('survminer')
#first edit the status of patients with OS > AUC max time. (censoring status=0 (Alive), OS=365*5 days)
risk_score_table_multi_cox2$censoring_status[which(risk_score_table_multi_cox2$overall_survival > AUC_max_time)] <- 0
risk_score_table_multi_cox2$overall_survival[which(risk_score_table_multi_cox2$overall_survival > AUC_max_time)] <- AUC_max_time
fit_km <- survfit(Surv(overall_survival, censoring_status) ~high_low, data = risk_score_table_multi_cox_cgga)     
ggsurvplot(fit_km, conf.int = F,pval = T,legend.title="risk score group",
           legend.labs=c(paste0("high risk"), paste0("low risk")), 
           risk.table = T, 
           palette = c('red','blue'), surv.median.line = 'hv')













